# Autogenerated by Astropy-affiliated package astropy_helpers's setup.py on 2015-02-15 18:03:36.494515
from __future__ import unicode_literals
import datetime

version = "0.4.5"
githash = "None"


major = 0
minor = 4
bugfix = 5

release = True
timestamp = datetime.datetime(2015, 2, 15, 18, 3, 36, 494515)
debug = False

try:
    from ._compiler import compiler
except ImportError:
    compiler = "unknown"

try:
    from .cython_version import cython_version
except ImportError:
    cython_version = "unknown"
